#include <list>
#include <boost/bind.hpp>
#include <boost/thread.hpp>
#include <boost/thread/recursive_mutex.hpp>
#include <boost/enable_shared_from_this.hpp>

class ActiveObject : public boost::enable_shared_from_this<ActiveObject>
{
private:
    boost::recursive_mutex mActionLock;

    void action()
    {
        boost::recursive_mutex::scoped_lock asyncLock(mActionLock);
    }
public:
    void startAction()
    {
        boost::recursive_mutex::scoped_lock syncLock(mActionLock);

        boost::thread th(boost::bind(&ActiveObject::action, shared_from_this()));
    }
};

int main()
{
    {
        std::list< boost::shared_ptr<ActiveObject> > jobList;

        for (int nrObjects = 0; nrObjects < 1000; ++nrObjects)
        {
            boost::shared_ptr<ActiveObject> pActiveObject( new ActiveObject() );
            jobList.push_back(pActiveObject);

            for (int nrActions = 0; nrActions < 3; ++nrActions)
            {
                pActiveObject->startAction();
            }
        }
        // At this point, a lot of handles are in use.
    }
    // Here, the handles are released again.
    exit(0);
}
